@@ -6,22 +6,22 @@ package MarpaX::Languages::ECMAScript::AST::Grammar;
 # ABSTRACT: ECMAScript grammar written in Marpa BNF
 
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5;
-use Carp qw/croak/;
+use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
 
-our $VERSION = '0.002'; # TRIAL VERSION
+our $VERSION = '0.006'; # TRIAL VERSION
 
 
 sub new {
-  my $class = shift;
-  my $grammarName = shift;
+  my ($class, $grammarName, %grammarSpecificOptions) = @_;
 
   my $self = {};
   if (! defined($grammarName)) {
-    croak 'Usage: new($grammar_Name)';
+    InternalError(error => 'Usage: new($grammar_Name)');
   } elsif ($grammarName eq 'ECMAScript-262-5') {
-    $self->{_grammar} = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5->new(@_);
+    $self->{_grammarAlias} = 'ECMAScript_262_5';
+    $self->{_grammar} = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5->new(%grammarSpecificOptions);
   } else {
-    croak "Unsupported grammar name $grammarName";
+    InternalError(error => "Unsupported grammar name $grammarName");
   }
   bless($self, $class);
 
@@ -31,7 +31,31 @@ sub new {
 
 sub program {
     my ($self) = @_;
-    return $self->{_grammar}->program();
+    return $self->{_grammar}->program;
+}
+
+
+sub grammarAlias {
+    my ($self) = @_;
+    return $self->{_grammarAlias};
+}
+
+
+sub template {
+    my ($self) = @_;
+    return $self->{_grammar}->template;
+}
+
+
+sub stringNumericLiteral {
+    my ($self) = @_;
+    return $self->{_grammar}->stringNumericLiteral;
+}
+
+
+sub pattern {
+    my ($self) = @_;
+    return $self->{_grammar}->pattern;
 }
 
 
@@ -49,7 +73,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar - ECMAScript grammar written in Marp
 
 =head1 VERSION
 
-version 0.002
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -71,7 +95,7 @@ ECMAScript-262-5. The ECMAScript-262, Edition 5, as of L<http://www.ecma-interna
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new($class, $grammarName)
+=head2 new($class, $grammarName, %grammarSpecificOptions)
 
 Instance a new object. Takes the name of the grammar as argument. Remaining arguments are passed to the sub grammar method. Supported grammars are:
 
@@ -99,6 +123,22 @@ A MarpaX::Languages::ECMAScript::AST::Impl object
 
 =back
 
+=head2 grammarAlias($self)
+
+Returns the grammar alias, i.e. the one really used in this distribution.
+
+=head2 template($self)
+
+Returns the generic template associated to grammarName.
+
+=head2 stringNumericLiteral($self)
+
+Returns the generic stringNumericLiteral associated to grammarName.
+
+=head2 pattern($self)
+
+Returns the generic pattern associated to grammarName.
+
 =head1 SEE ALSO
 
 L<Marpa::R2>
