@@ -6,12 +6,11 @@ use MarpaX::Languages::ECMAScript::AST::Util qw/:all/;
 use MarpaX::Languages::ECMAScript::AST::Impl qw//;
 use Log::Any qw/$log/;
 use constant SEARCH_KEYWORD_IN_GRAMMAR => '# DO NOT REMOVE NOR MODIFY THIS LINE';
-
-use Carp qw/croak/;
+use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
 
 # ABSTRACT: ECMAScript, grammars base package
 
-our $VERSION = '0.002'; # TRIAL VERSION
+our $VERSION = '0.006'; # TRIAL VERSION
 
 #
 # Note: because this module is usually subclasses, internal methods are called
@@ -20,24 +19,18 @@ our $VERSION = '0.002'; # TRIAL VERSION
 
 
 sub new {
-  my ($class, $grammar, $package, $spec) = @_;
+  my ($class, $grammar_content, $package, $spec) = @_;
 
-  croak "Missing grammar" if (! defined($grammar));
-  croak "Missing package name" if (! defined($package));
-  croak "Missing ECMAScript specification" if (! defined($spec));
+  InternalError(error => 'Missing grammar') if (! defined($grammar_content));
+  InternalError(error => 'Missing package name') if (! defined($package));
+  InternalError(error => 'Missing ECMAScript specification') if (! defined($spec));
 
   my $self  = {
-      _content => $grammar,
-      _grammar_option => {action_object  => sprintf('%s::%s', $package, 'Actions')},
-      _recce_option => {ranking_method => 'high_rule_only'},
+      _content => $class->make_content($spec, $grammar_content),
+      _grammar_option => $class->make_grammar_option($package, $spec, $grammar_content),
+      _recce_option => $class->make_recce_option,
       _strict => 0
   };
-  #
-  # Too painful to write MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::IsSomething
-  # so I change it on-the-fly here
-  #
-  my $characterClass = "\\p{MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::Is";
-  $self->{_content} =~ s/\\p\{Is/$characterClass/g;
 
   bless($self, $class);
 
@@ -60,6 +53,24 @@ sub content {
 }
 
 
+sub make_content {
+    my ($class, $spec, $original_content) = @_;
+
+    #
+    # Too painful to write MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::IsSomething
+    # so I change it on-the-fly here
+    #
+    if ($spec eq 'ECMAScript-262-5') {
+	$spec = 'ECMAScript_262_5';
+    }
+    my $characterClass = "\\p{MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::Is";
+    my $content = $original_content;
+    $content =~ s/\\p\{Is/$characterClass/g;
+
+    return $content;
+}
+
+
 sub extract {
     my ($self) = @_;
     my $rc = '';
@@ -76,6 +87,13 @@ sub extract {
 }
 
 
+sub make_grammar_option {
+    my ($class, $package, $spec, $grammar_content) = @_;
+    $package //= $class;
+    return {bless_package => $class, action_object  => sprintf('%s::%s', $package, 'Actions'), source => \$class->make_content($spec, $grammar_content)};
+}
+
+
 sub grammar_option {
     my ($self) = @_;
     return $self->{_grammar_option};
@@ -88,6 +106,12 @@ sub recce_option {
 }
 
 
+sub make_recce_option {
+    my ($class) = @_;
+    return {ranking_method => 'high_rule_only'};
+}
+
+
 sub _callback {
   my ($self, $source, $pos, $max, $impl, $callbackp, $originalErrorString, @args) = @_;
 
@@ -98,17 +122,22 @@ sub _callback {
     my $callackErrorString = $@;
     my $line_columnp;
     eval {$line_columnp = lineAndCol($impl)};
+    my $context = _context($self, $impl);
+    #
+    # Now we can destroy the recognizer
+    #
+    $impl->destroy_R;
     if (! $@) {
       if (defined($originalErrorString) && $originalErrorString) {
-        logCroak("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), $context));
       } else {
-        logCroak("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), $context));
       }
     } else {
       if (defined($originalErrorString) && $originalErrorString) {
-        logCroak("%s\n%s\n%s", $originalErrorString, $callackErrorString, _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s\n%s", $originalErrorString, $callackErrorString, $context));
       } else {
-        logCroak("%s\n%s", $callackErrorString, _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s", $callackErrorString, $context));
       }
     }
   }
@@ -129,21 +158,29 @@ sub parse {
   my $endp = $optionsp->{end};
   my $endargsp = $optionsp->{endargs} // [];
   my @endargs = @{$endargsp};
+  my $keepOriginalSource = $optionsp->{keepOriginalSource} // 1;
+
   $start //= 0;
   $length //= -1;
 
   #
   # This will create a new instance of the string
   #
-  $source .= ' ';
+  if (! $keepOriginalSource) {
+      #
+      # Space for an eventual last and inserted semicolon
+      #
+      $source .= ' ';
+  }
 
   my $pos = $start;
   my $max = length($source) - $start + $length;
   my $stop;
   my $newpos;
   #
-  # Space for an eventual last and inserted semicolon
+  # Create a recognizer
   #
+  $impl->make_R;
   #
   # Lexer can fail
   #
@@ -156,7 +193,9 @@ sub parse {
       $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
     } else {
       my $line_columnp = lineAndCol($impl);
-      logCroak("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+      my $context = _context($self, $impl);
+      $impl->destroy_R;
+      SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), $context));
     }
   } else {
     $pos = $newpos;
@@ -180,7 +219,9 @@ sub parse {
         $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
       } else {
         my $line_columnp = lineAndCol($impl);
-        logCroak("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+	my $context = _context($self, $impl);
+	$impl->destroy_R;
+        SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), $context));
       }
     } else {
       $pos = $newpos;
@@ -201,13 +242,17 @@ sub parse {
 sub value {
   my ($self, $impl) = @_;
 
-  my $rc = $impl->value() || logCroak('%s', _show_last_expression($self, $impl));
+  my $rc = $impl->value() || do {$impl->destroy_R; InternalError(error => sprintf('%s', _show_last_expression($self, $impl)))};
   if (! defined($rc)) {
-      croak "Undefined parse tree value";
+      $impl->destroy_R;
+      InternalError(error => 'Undefined parse tree value');
   }
   if (defined($impl->value())) {
-      croak "More than one parse tree value\n";
+      $impl->destroy_R;
+      InternalError(error => 'More than one parse tree value');
   }
+  $impl->destroy_R;
+
   return ${$rc};
 }
 
@@ -293,7 +338,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::Base - ECMAScript, grammars base pa
 
 =head1 VERSION
 
-version 0.002
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -313,9 +358,9 @@ This modules returns a base package for all the ECMAScript grammars written in M
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new($grammar, $package, $spec)
+=head2 new($grammar_content, $package, $spec)
 
-Instance a new object. Takes a grammar, a package name and an ECMAScript specification as required parameters.
+Instance a new object. Takes a grammar content $grammar_content, a package name $package and an ECMAScript specification $spec as required parameters.
 
 =head2 strict($self, [$strict])
 
@@ -325,10 +370,18 @@ Sets/Returns the strict mode of the grammar.
 
 Returns the content of the grammar.
 
+=head2 make_content($class, $spec, $original_content)
+
+Class method that return the default content of the grammar writen for specification $spec and based on $original_content. Grammars in the ECMA script typically use Posix user-defined classes without the full classname; this method is making sure full classname is used.
+
 =head2 extract($self)
 
 Returns the part of the grammar that can be safely extracted and injected in another.
 
+=head2 make_grammar_option($class, $package, $spec, $grammar_content)
+
+Class method that returns default grammar options for a given package $package, ECMA specification $spec, and grammar content $grammar_content. Default $package is the class name used for this call.
+
 =head2 grammar_option($self)
 
 Returns recommended option for Marpa::R2::Scanless::G->new(), returned as a reference to a hash.
@@ -337,6 +390,10 @@ Returns recommended option for Marpa::R2::Scanless::G->new(), returned as a refe
 
 Returns recommended option for Marpa::R2::Scanless::R->new(), returned as a reference to a hash.
 
+=head2 make_recce_option($class, $package)
+
+Class method that returns default recce options.
+
 =head2 parse($self, $source, [$optionsp], [$start], [$length])
 
 Parse the source given as reference to a scalar, an optional reference to a options that is a hash that can contain:
@@ -345,35 +402,39 @@ Parse the source given as reference to a scalar, an optional reference to a opti
 
 =item callbackargsp
 
-Callbak Code Reference
+Callbak Code Reference. Default is undef.
 
 =item callbackargs
 
-Reference to an array of Callback Code Reference first arguments
+Reference to an array of Callback Code Reference first arguments. Default is [].
 
 =item failure
 
-Failure callback Code Reference
+Failure callback Code Reference. Default is undef.
 
 =item failureargs
 
-Reference to an array of Failure callback Code Reference first arguments
+Reference to an array of Failure callback Code Reference first arguments. Default is [].
 
 =item end
 
-End callback Code Reference
+End callback Code Reference. Default is undef.
 
 =item endargs
 
-Reference to an array of End callback Code Reference first arguments
+Reference to an array of End callback Code Reference first arguments. Default is [].
+
+=item keepOriginalSource
+
+Because of Automatic Semicolon Insertion that may happen at the end, a space is appended to a copy of the source to be parsed. If a true value, this option disable that append. Default is true.
 
 =back
 
-This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and croak if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored.
+This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and raise a MarpaX::Languages::ECMAScript::AST::Exception::SyntaxError exception if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored. Please note that this method explicitely creates a recognizer using $impl->make_R(), destroyed in case of error.
 
 =head2 value($self, $impl)
 
-Return the blessed value. $impl is the recognizer instance for the grammar. Will croak if there is more than one parse tree value.
+Return the parse tree (unique) value. $impl is the recognizer instance for the grammar. Will raise an InternalError exception if there is no parse tree value, or more than one parse tree value. Please note that this method explicity destroys the recognizer using $impl->destroy_R. Value itself is an AST where every string is a perl string.
 
 =head2 getLexeme($self, $lexemeHashp, $impl)
 
@@ -401,7 +462,7 @@ Line number as per Marpa
 
 Column number as per Marpa
 
-=value
+=item value
 
 Lexeme value
 
@@ -423,7 +484,7 @@ Start position
 
 Length
 
-=value
+=item value
 
 Lexeme value
 
