@@ -11,17 +11,19 @@ use warnings FATAL => 'all';
 
 package MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program;
 use parent qw/MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Base/;
+use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program::Singleton;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program::Actions;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::RegularExpressionLiteral;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::StringLiteral;
+use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::NumericLiteral;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses;
-use Carp qw/croak/;
+use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
 use Log::Any qw/$log/;
 use SUPER;
 
 # ABSTRACT: ECMAScript-262, Edition 5, lexical program grammar written in Marpa BNF
 
-our $VERSION = '0.002'; # TRIAL VERSION
+our $VERSION = '0.006'; # TRIAL VERSION
 
 our $WhiteSpace        = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsWhiteSpace}])/;
 our $LineTerminator    = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsLineTerminator}])/;
@@ -88,9 +90,20 @@ map {$grammar_source .= uc($_) . " ~ '$_'\n"} @BooleanLiteral;
 #
 our $StringLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::StringLiteral->new();
 our $RegularExpressionLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::RegularExpressionLiteral->new();
+our $NumericLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::NumericLiteral->new();
 $grammar_source .= $StringLiteral->extract;
+$grammar_source .= $NumericLiteral->extract;
 $grammar_source .= $RegularExpressionLiteral->extract;
 
+our $singleton = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program::Singleton->instance(
+    MarpaX::Languages::ECMAScript::AST::Impl->new
+    (
+     __PACKAGE__->make_grammar_option(__PACKAGE__, 'ECMAScript-262-5', $grammar_source),
+     undef,                                   # $recceOptionsHashp
+     undef,                                   # $cachedG
+     1                                        # $noR
+    )->grammar
+    );
 
 #
 # For convenience in the IDENTIFIER$ lexeme callback, we merge Keyword, FutureReservedWord, NullLiteral, BooleanLiteral into
@@ -109,6 +122,11 @@ sub new {
 }
 
 
+sub G {
+    return $singleton->G;
+}
+
+
 sub parse {
     my ($self, $source, $impl) = @_;
     $self->{programCompleted} = 0;
@@ -120,6 +138,7 @@ sub parse {
                          failureargs => [ $self ],
                          end => \&_endCallback,
                          endargs => [ $self ],
+			 keepOriginalSource => 0,
                         });
 }
 
@@ -136,7 +155,9 @@ sub _eventCallback {
     #
     # Events are always in this order:
     #
+    # ---------------------------------
     # 1. Completion events first (XXX$)
+    # ---------------------------------
     #
     if ($name eq 'Program$') {
 	#
@@ -153,20 +174,36 @@ sub _eventCallback {
           $self->_isDecimalDigit($source, $pos, $impl)) {
         my ($start, $end) = $impl->last_completed_range('NumericLiteral');
         my $lastNumericLiteral = $impl->range_to_string($start, $end);
-        croak "[_eventCallback] NumericLiteral $lastNumericLiteral must not be immediately followed by an IdentifierStart or DecimalDigit";
+        SyntaxError(error => "NumericLiteral $lastNumericLiteral must not be immediately followed by an IdentifierStart or DecimalDigit");
       }
     }
-    # 2. Then nulled events (XXX[])
-    # 3. Then prediction events (^XXX or ^^XXX)
-    #
     elsif ($name eq 'IDENTIFIER$') {
 	my %lastLexeme = ();
 	$self->getLastLexeme(\%lastLexeme, $impl);
 	if (exists($ReservedWord{$lastLexeme{value}})) {
-	    croak "[_eventCallback] Identifier $lastLexeme{value} is a reserved word";
+	    SyntaxError(error => "Identifier $lastLexeme{value} is a reserved word");
 	}
     }
     #
+    # ------------------------
+    # 2. nulled events (XXX[])
+    # ------------------------
+    #
+    # ------------------------------------
+    # 3. prediction events (^XXX or ^^XXX)
+    # ------------------------------------
+    #
+    elsif ($name eq '^INVISIBLE_SEMICOLON') {
+      #
+      # In the AST, we explicitely associate the ';' to the missing semicolon
+      #
+	my %lastLexeme = ();
+	$self->getLastLexeme(\%lastLexeme, $impl);
+	my $Slength = $self->_preSLength($source, $rc, $impl);
+	$self->_insertInvisibleSemiColon($impl, $rc, $Slength);
+	$rc += $Slength;
+    }
+    #
     # ^PLUSPLUS_POSTFIX, ^MINUSMINUS_POSTFIX
     # --------------------------------------
     elsif ($name eq '^PLUSPLUS_POSTFIX' || $name eq '^MINUSMINUS_POSTFIX') {
@@ -175,14 +212,11 @@ sub _eventCallback {
       my $postLineTerminatorPos = $lastLexeme{start} + $lastLexeme{length};
       my $postLineTerminatorLength = $self->_postLineTerminatorLength($source, $postLineTerminatorPos, $impl);
       if ($postLineTerminatorLength > 0) {
-	  #$log->infof('[_eventCallback] Automatic Semicolon Insertion at position %d, span %d', $postLineTerminatorPos, $postLineTerminatorLength);
-	  #$log->tracef('[_eventCallback] Event %s: lexeme_read(\'SEMICOLON\', %d, %d, \';\')', $name, $postLineTerminatorPos, $postLineTerminatorLength);
 	  $impl->lexeme_read('SEMICOLON', $postLineTerminatorPos, $postLineTerminatorLength, ';');
       }
       my $lname = $name;
       substr($lname, 0, 1, '');
       my $lvalue = ($lname eq 'PLUSPLUS_POSTFIX') ? '++' : '--';
-      #$log->tracef('[_eventCallback] Event %s: lexeme_read(\'%s\', %d, %d, \'%s\')', $name, $lname, $rc, 2, $lvalue);
       $impl->lexeme_read($lname, $rc, 2, $lvalue);
       $rc += 2;
     }
@@ -195,7 +229,6 @@ sub _eventCallback {
           index($source, '/=', $realpos) != $realpos &&
           index($source, '//', $realpos) != $realpos &&
           index($source, '/*', $realpos) != $realpos) {
-        #$log->tracef('[_eventCallback] Event %s: lexeme_read(\'DIV\', %d, 1, \'/\')', $name, $realpos);
         $impl->lexeme_read('DIV', $realpos, 1, '/');
         $rc = $realpos + 1;
       }
@@ -208,7 +241,6 @@ sub _eventCallback {
       if (index($source, '/=', $realpos) == $realpos &&
           index($source, '//', $realpos) != $realpos &&
           index($source, '/*', $realpos) != $realpos) {
-        #$log->tracef('[_eventCallback] Event %s: lexeme_read(\'DIVASSIGN\', %d, 2, \'/=\')', $name, $realpos);
         $impl->lexeme_read('DIVASSIGN', $realpos, 2, '/=');
         $rc = $realpos + 2;
       }
@@ -361,10 +393,16 @@ sub _isEnd {
 sub _insertSemiColon {
   my ($self, $impl, $pos, $length) = @_;
 
-  #$log->tracef('[_insertSemiColon] Automatic Semicolon Insertion at position %d, length %d', $pos, $length);
-  #$log->tracef('[_insertSemiColon] lexeme_read(\'SEMICOLON\', %d, %d, \';\')', $pos, $length);
   if (! $impl->lexeme_read('SEMICOLON', $pos, $length, ';')) {
-    croak "[_insertSemiColon] Automatic Semicolon Insertion not allowed at position $pos";
+    SyntaxError(error => "Automatic Semicolon Insertion not allowed at position $pos");
+  }
+}
+
+sub _insertInvisibleSemiColon {
+  my ($self, $impl, $pos, $length) = @_;
+
+  if (! $impl->lexeme_read('INVISIBLE_SEMICOLON', $pos, $length, ';')) {
+    SyntaxError(error => "Automatic Invisible Semicolon Insertion not allowed at position $pos");
   }
 }
 
@@ -395,11 +433,9 @@ sub _failureCallback {
   } elsif ($self->_isRcurly($source, $rc, $impl)) {
     $self->_insertSemiColon($impl, $rc, 1);
   } else {
-    croak "[_failureCallback] Failure at position $rc: '" . substr($source, $rc, 10) . "'";
+    SyntaxError();
   }
 
-  #$log->tracef('[_failureCallback] Resuming at position %d', $rc);
-
   return $rc;
 }
 
@@ -424,7 +460,7 @@ sub _endCallback {
       $self->_insertSemiColon($impl, $lastValidPos, 1);
       my $haveProgramCompletion = grep {$_ eq 'Program$'} map {$_->[0]} @{$impl->events};
       if (! $haveProgramCompletion) {
-	  croak "end callback failure";
+	  SyntaxError(error => "Incomplete program");
       }
   }
 }
@@ -442,7 +478,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program - ECMAScr
 
 =head1 VERSION
 
-version 0.002
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -466,9 +502,13 @@ This modules returns describes the ECMAScript 262, Edition 5 lexical program gra
 
 Instance a new object.
 
-=head2 parse($self, $source)
+=head2 G()
 
-Parse the source given as $source.
+Cached Marpa::R2::Scanless::G compiled grammar.
+
+=head2 parse($self, $source, $impl)
+
+Parse the source given as $source using implementation $impl.
 
 =head1 SEE ALSO
 
@@ -496,7 +536,7 @@ __DATA__
 # tokens, line terminators, comments, or white space.
 #
 :start ::= Program
-:default ::= action => [values] bless => ::lhs
+:default ::= action => valuesAndRuleId # bless => ::lhs
 lexeme default = action => [start,length,value]
 
 #
@@ -509,15 +549,6 @@ Literal ::=
   | StringLiteral
   | RegularExpressionLiteral
 
-#
-# NumericLiteral definition as per lexical grammar
-#
-event 'NumericLiteral$' = completed <NumericLiteral>
-NumericLiteral ::=
-    DECIMALLITERAL           action => DecimalLiteral
-  | HEXINTEGERLITERAL        action => HexIntegerLiteral
-  | OCTALINTEGERLITERAL      action => OctalIntegerLiteral
-
 PrimaryExpression ::=
     THIS
   | IDENTIFIER
@@ -830,19 +861,16 @@ ContinueStatement ::=
     CONTINUE           SEMICOLON
   | CONTINUE INVISIBLE_SEMICOLON
   | CONTINUE IDENTIFIER           SEMICOLON
-#  | CONTINUE IDENTIFIER INVISIBLE_SEMICOLON
 
 BreakStatement ::=
     BREAK           SEMICOLON
   | BREAK INVISIBLE_SEMICOLON
   | BREAK IDENTIFIER           SEMICOLON
-#  | BREAK IDENTIFIER INVISIBLE_SEMICOLON
 
 ReturnStatement ::=
     RETURN           SEMICOLON
   | RETURN INVISIBLE_SEMICOLON
   | RETURN Expression           SEMICOLON
-#  | RETURN Expression INVISIBLE_SEMICOLON
 
 WithStatement ::=
     WITH  LPAREN  Expression  RPAREN  Statement
@@ -875,7 +903,6 @@ LabelledStatement ::=
 
 ThrowStatement ::=
       THROW Expression SEMICOLON
-#    | THROW Expression INVISIBLE_SEMICOLON
 
 TryStatement ::=
     TRY  Block  Catch
@@ -957,12 +984,20 @@ _S_ANY ~ _S*
 #   The subtility is that when INVISIBLE_SEMICOLON matches, we know per-def this is an automatic
 #   semicolon insertion: grammar expected a semicolon, and found it hidden.
 #
+:lexeme ~ <INVISIBLE_SEMICOLON> pause => before event => '^INVISIBLE_SEMICOLON'
 INVISIBLE_SEMICOLON ~ _S_ANY _SLT _S_ANY
 
 NullLiteral                           ::= NULL
 BooleanLiteral                        ::= TRUE | FALSE
-StringLiteral                         ::= STRINGLITERAL
+StringLiteral                         ::= STRINGLITERAL           action => StringLiteral
 RegularExpressionLiteral              ::= REGULAREXPRESSIONLITERAL
+NumericLiteral                        ::= DecimalLiteral
+                                        | HexIntegerLiteral
+                                        | OctalIntegerLiteral
+DecimalLiteral                        ::= DECIMALLITERAL
+HexIntegerLiteral                     ::= HEXINTEGERLITERAL
+OctalIntegerLiteral                   ::= OCTALINTEGERLITERAL
+event 'NumericLiteral$' = completed <NumericLiteral>
 
 # --------------------------------------------------------------------------------------
 #
@@ -1175,58 +1210,6 @@ _HexDigit                              ~ [\p{IsHexDigit}]
 :lexeme ~ <IDENTIFIER>  pause => after event => 'IDENTIFIER$'
 IDENTIFIER                             ~ _IdentifierNameInternal
 
-# --------------
-# DECIMALLITERAL
-# --------------
-DECIMALLITERAL                         ~ _DecimalIntegerLiteral '.' _DecimalDigitsopt _ExponentPartopt
-                                       | '.' _DecimalDigits _ExponentPartopt
-                                       | _DecimalIntegerLiteral _ExponentPartopt
-
-_DecimalDigitsopt                      ~ _DecimalDigits
-_DecimalDigitsopt                      ~
-
-_ExponentPartopt                       ~ _ExponentPart
-_ExponentPartopt                       ~
-
-_DecimalIntegerLiteral                 ~ '0'
-                                       | __NonZeroDigit _DecimalDigitsopt
-
-_DecimalDigits                         ~ __DecimalDigit
-                                       | _DecimalDigits __DecimalDigit
-
-_ExponentPart                          ~ __ExponentIndicator _SignedInteger
-
-_SignedInteger                         ~ _DecimalDigits
-                                       | '+' _DecimalDigits
-                                       | '-' _DecimalDigits
-
-__ExponentIndicator                    ~ [\p{IsExponentIndicator}]
-
-__NonZeroDigit                         ~ [\p{IsNonZeroDigit}]
-
-__DecimalDigit                         ~ [\p{IsDecimalDigit}]
-
-# -----------------
-# HEXINTEGERLITERAL
-# ----------------
-HEXINTEGERLITERAL                      ~ _HexIntegerLiteralInternal
-
-_HexIntegerLiteralInternal             ~ '0x' _HexDigit
-                                       | '0X' _HexDigit
-                                       | _HexIntegerLiteralInternal _HexDigit
-
-# Note: see higher for definition of __HexDigit
-
-# -------------------
-# OCTALINTEGERLITERAL
-# -------------------
-OCTALINTEGERLITERAL                    ~ _OctalIntegerLiteralInternal
-
-_OctalIntegerLiteralInternal           ~ '0' __OctalDigit
-                                       | _OctalIntegerLiteralInternal __OctalDigit
-
-__OctalDigit                           ~ [\p{IsOctalDigit}]
-
 # -----------
 # _WhiteSpace
 # -----------
@@ -1295,6 +1278,15 @@ STRINGLITERAL                          ~ __StringLiteral
 # --------------------------------------------------------------------------------------
 REGULAREXPRESSIONLITERAL               ~ __RegularExpressionLiteral
 
+# --------------------------------------------------------------------------------------
+# __DecimalLiteral      injected: it is a G1 grammar in NumericLiteral.pm
+# __HexIntegerLiteral   injected: it is a G1 grammar in NumericLiteral.pm
+# __OctalIntegerLiteral injected: it is a G1 grammar in NumericLiteral.pm
+# --------------------------------------------------------------------------------------
+DECIMALLITERAL                         ~ __DecimalLiteral
+HEXINTEGERLITERAL                      ~ __HexIntegerLiteral
+OCTALINTEGERLITERAL                    ~ __OctalIntegerLiteral
+
 # -------------------------------------------
 # Injection of reserved keywords happens here
 # -------------------------------------------
