@@ -10,7 +10,7 @@ our $grammar_content = do {local $/; <DATA>};
 
 # ABSTRACT: ECMAScript-262, Edition 5, grammar written in Marpa BNF
 
-our $VERSION = '0.002'; # TRIAL VERSION
+our $VERSION = '0.006'; # TRIAL VERSION
 
 
 sub new {
@@ -62,7 +62,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Uri - ECMAScript-
 
 =head1 VERSION
 
-version 0.002
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -223,7 +223,7 @@ ClassAtom ~
     | ClassAtomNoDash
 
 ClassAtomNoDash ~
-      [\p{IsSourceCharacterButNotOneOfBackslashOrRbracketorMinus}]
+      [\p{IsSourceCharacterButNotOneOfBackslashOrRbracketOrMinus}]
     | '\' ClassEscape
 # '
 ClassEscape ~
