@@ -8,14 +8,13 @@ package MarpaX::Languages::ECMAScript::AST::Util;
 use Exporter 'import';
 use Log::Any qw/$log/;
 use Data::Dumper;
-use Carp qw/croak/;
 # Marpa follows Unicode recommendation, i.e. perl's \R, that cannot be in a character class
 our $NEWLINE_REGEXP = qr/(?>\x0D\x0A|\v)/;
 
-our $VERSION = '0.002'; # TRIAL VERSION
+our $VERSION = '0.006'; # TRIAL VERSION
 # CONTRIBUTORS
 
-our @EXPORT_OK = qw/whoami whowasi traceAndUnpack logCroak showLineAndCol lineAndCol lastCompleted startAndLength lastLexemeSpan/;
+our @EXPORT_OK = qw/whoami whowasi traceAndUnpack showLineAndCol lineAndCol lastCompleted startAndLength lastLexemeSpan/;
 our %EXPORT_TAGS = ('all' => [ @EXPORT_OK ]);
 
 
@@ -65,25 +64,6 @@ sub traceAndUnpack {
 }
 
 
-sub logCroak {
-    my ($fmt, @arg) = @_;
-
-    my $msg = sprintf($fmt, @arg);
-    $log->fatalf($msg);
-    if (! $log->is_fatal()) {
-      #
-      # Logging is not enabled at FATAL level: re do the message in croak
-      #
-      croak $msg;
-    } else {
-      #
-      # Logging is enabled at FATAL level: no new message
-      #
-      croak;
-    }
-}
-
-
 sub showLineAndCol {
     my ($line, $col, $source) = @_;
 
@@ -159,7 +139,7 @@ MarpaX::Languages::ECMAScript::AST::Util - ECMAScript Translation to AST - Class
 
 =head1 VERSION
 
-version 0.002
+version 0.006
 
 =head1 SYNOPSIS
 
@@ -195,10 +175,6 @@ Returns the name of the parent's calling routine. Optional $base prefix is remov
 
 Returns a hash mapping @{$nameOfArgumentsp} to @arguments and trace it. The tracing is done using a method quite similar to Log::Any. Tracing and hash mapping stops at the end of @nameOfArguments or @arguments.
 
-=head2 logCroak($fmt, @arg)
-
-Formats a string using Log::Any, issue a $log->fatal with it, and croak with it.
-
 =head2 showLineAndCol($line, $col, $source)
 
 Returns a string showing the request line, followed by another string that shows what is the column of interest, in the form "------^".
